@@ -26,6 +26,8 @@ import gtk
 import gobject
 
 from hamster import dispatcher, storage, SHARED_DATA_DIR
+from hamster.stuff import *
+
 import hamster.eds
 
 import time
@@ -36,21 +38,10 @@ GLADE_FILE = "add_custom_fact.glade"
 
 class CustomFactController:
     def __init__(self,  fact_date = None, fact_id = None):
-        self.wTree = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, GLADE_FILE))
+        self.glade = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, GLADE_FILE))
         self.window = self.get_widget('custom_fact_window')
 
-        # load window of activity switcher and todays view
-        self.items = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
-        self.activities = gtk.ListStore(gobject.TYPE_STRING)
-        self.completion = gtk.EntryCompletion()
-        self.completion.set_model(self.activities)
-        self.completion.set_text_column(0)
-        self.completion.set_minimum_key_length(1) 
-        activity_list = self.wTree.get_widget('activity-list')
-        activity_list.set_model(self.items)
-        activity_list.set_text_column(0)
-        activity_list.child.set_completion(self.completion)
-
+        self.set_dropdown()
 
         self.hours = gtk.ListStore(gobject.TYPE_STRING)
         
@@ -79,6 +70,11 @@ class CustomFactController:
             self.get_widget('start_time').set_text("%02d:%02d" % (fact["start_time"].hour, fact["start_time"].minute))
 
             self.get_widget('activity_name').set_text(fact["name"])
+            
+            buf = gtk.TextBuffer()
+            buf.set_text(fact["description"] or "")
+            self.get_widget('description').set_buffer(buf)
+
             self.get_widget("ok").set_sensitive(True)
             self.get_widget("ok").set_label("gtk-save")
             self.window.set_title(_("Update activity"))
@@ -96,36 +92,92 @@ class CustomFactController:
                 self.get_widget('duration_mins').set_value(for_minutes)
 
 
-        self.wTree.signal_autoconnect(self)
-        
-    def refresh_menu(self):
-        all_activities = storage.get_activities()
-        self.activities.clear()
-        for activity in all_activities:
-            self.activities.append([activity['name']])
+        self.glade.signal_autoconnect(self)
 
-        activity_list = self.get_widget('activity-list')
-        store = activity_list.get_model()
+    def set_dropdown(self):
+        # set up drop down menu
+        self.activity_list = self.glade.get_widget('activity-list')
+        self.activity_list.set_model(gtk.ListStore(gobject.TYPE_STRING,
+                                                   gobject.TYPE_STRING,
+                                                   gobject.TYPE_STRING))
+
+        self.activity_list.clear()
+        activity_cell = gtk.CellRendererText()
+        self.activity_list.pack_start(activity_cell, True)
+        self.activity_list.add_attribute(activity_cell, 'text', 0)
+        category_cell = CategoryCell()  
+        self.activity_list.pack_start(category_cell, False)
+        self.activity_list.add_attribute(category_cell, 'text', 1)
+        
+        self.activity_list.set_property("text-column", 2)
+
+
+        # set up autocompletition
+        self.activities = gtk.ListStore(gobject.TYPE_STRING,
+                                        gobject.TYPE_STRING,
+                                        gobject.TYPE_STRING)
+        completion = gtk.EntryCompletion()
+        completion.set_model(self.activities)
+
+        activity_cell = gtk.CellRendererText()
+        completion.pack_start(activity_cell, True)
+        completion.add_attribute(activity_cell, 'text', 0)
+        completion.set_property("text-column", 2)
+
+        category_cell = CategoryCell()  
+        completion.pack_start(category_cell, False)
+        completion.add_attribute(category_cell, 'text', 1)
+
+        def match_func(completion, key, iter):
+            model = completion.get_model()
+            text = model.get_value(iter, 2)
+            if text and text.startswith(key):
+                return True
+            return False
+
+        completion.set_match_func(match_func)
+        completion.set_minimum_key_length(1)
+        completion.set_inline_completion(True)
+
+        self.activity_list.child.set_completion(completion)
+
+
+    def refresh_menu(self):
+        #first populate the autocomplete - contains all entries in lowercase
+        self.activities.clear()
+        all_activities = storage.get_autocomplete_activities()
+        for activity in all_activities:
+            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            self.activities.append([activity['name'],
+                                    activity['category'],
+                                    activity_category])
+
+
+        #now populate the menu - contains only categorized entries
+        store = self.activity_list.get_model()
         store.clear()
 
         #populate fresh list from DB
-        activities = storage.get_sorted_activities()
-        prev_item = None
+        categorized_activities = storage.get_sorted_activities()
 
-        today = datetime.date.today()
-        for activity in activities:
-            item = store.append([activity['name'], activity['id']])
+        for activity in categorized_activities:
+            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            item = store.append([activity['name'],
+                                 activity['category'],
+                                 activity_category])
 
+        # finally add TODO tasks from evolution to both lists
         tasks = hamster.eds.get_eds_tasks()
         for activity in tasks:
-            item = store.append([activity['name'], -1])
-            
+            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            self.activities.append([activity['name'],activity['category'],activity_category])
+            store.append([activity['name'], activity['category'], activity_category])
 
         return True
 
     def get_widget(self, name):
         """ skip one variable (huh) """
-        return self.wTree.get_widget(name)
+        return self.glade.get_widget(name)
 
     def show(self):
         self.window.show()
@@ -164,6 +216,27 @@ class CustomFactController:
     
     def on_ok_clicked(self, button):
         activity = self.get_widget("activity-list").get_child().get_text()
+
+        # juggle with description - break into parts and then put together
+        buf = self.get_widget('description').get_buffer()
+        description = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), 0)
+        description = description.strip()
+        
+        # user might also type description in the activity name - strip it here
+        # and remember value
+        inline_description = None
+        if activity.find(",") != -1:
+            activity, inline_description  = activity.split(",", 1)
+            inline_description = inline_description.strip()
+        
+        # description field is prior to inline description
+        description = description or inline_description
+        
+        if description:
+            activity = "%s, %s" % (activity, description)
+
+        
+        
         start_time = self._get_datetime("start")
 
         end_time = None
