@@ -24,6 +24,9 @@ import datetime
 import os.path
 import gnomeapplet, gtk
 import gobject
+import dbus
+import dbus.service
+import dbus.mainloop.glib
 
 from hamster import dispatcher, storage, SHARED_DATA_DIR
 import hamster.eds
@@ -31,8 +34,34 @@ from hamster.Configuration import GconfStore
 
 from hamster.stuff import *
 from hamster.KeyBinder import *
+from hamster.hamsterdbus import HAMSTER_URI, HamsterDbusController 
+
 import idle
 
+try:
+    import pynotify
+    PYNOTIFY = True
+except:
+    PYNOTIFY = False
+    
+class Notifier(object):
+    def __init__(self, app_name, icon, attach):
+        self._icon = icon
+        self._attach = attach
+        self._notify = None
+        # Title of reminder baloon
+        self.summary = _("Time Tracker")
+      
+        if not pynotify.is_initted():
+            pynotify.init(app_name)
+
+    def msg(self, body, switch_cb, stop_cb):
+        self._notify = pynotify.Notification(self.summary, body, self._icon, self._attach)
+        self._notify.add_action("refresh", _("Switch Task"), switch_cb)
+        self._notify.add_action("no", _("Stop Tracking"), stop_cb)
+        self._notify.show()
+
+
 class PanelButton(gtk.ToggleButton):
     def __init__(self):
         gtk.ToggleButton.__init__(self)
@@ -40,58 +69,169 @@ class PanelButton(gtk.ToggleButton):
         self.set_border_width(0)
         
         self.label = gtk.Label()
+        self.label.set_justify(gtk.JUSTIFY_CENTER)
+
+        self.label.connect('style-set', self.on_label_style_set)
+        self.connect('size_allocate', self.on_size_allocate)
+
         self.add(self.label)
-
-    def set_text(self, text):
-        self.label.set_text(text)
-
-    def get_pos(self):
-        return gtk.gdk.Window.get_origin(self.label.window)
+        
+        self.activity, self.duration = None, None
+        self.prev_size = 0
 
     def set_active(self, is_active):
         self.set_property('active', is_active)
 
+    def set_text(self, activity, duration):
+        self.activity = activity
+        self.duration = duration
+        self.reformat_label()
+        
+    def reformat_label(self):
+        label = self.activity
+        if self.duration:
+            if self.use_two_line_format():
+                label = "%s\n%s" % (self.activity, self.duration)
+            else:
+                label = "%s %s" % (self.activity, self.duration)
+        
+        label = '<span gravity=\"south\">' + label + '</span>'
+        self.label.set_markup(label)
+
+    def get_pos(self):
+        return gtk.gdk.Window.get_origin(self.label.window)
+
+
+    def use_two_line_format(self):
+        if not self.get_parent():
+            return False
+
+        popup_dir = self.get_parent().get_orient()
+ 
+        orient_vertical = popup_dir in [gnomeapplet.ORIENT_LEFT] or \
+                          popup_dir in [gnomeapplet.ORIENT_RIGHT]
+
+
+        import pango
+        context = self.label.get_pango_context()
+        metrics = context.get_metrics(self.label.style.font_desc,
+                                      pango.Context.get_language(context))
+        ascent = pango.FontMetrics.get_ascent(metrics)
+        descent = pango.FontMetrics.get_descent(metrics)
+
+        if orient_vertical == False:
+            thickness = self.style.ythickness;
+        else:
+            thickness = self.style.xthickness;
+
+        focus_width = self.style_get_property("focus-line-width")
+        focus_pad = self.style_get_property("focus-padding")
+
+        required_size = 2 * ((pango.PIXELS(ascent + descent) ) + 2 * (focus_width + focus_pad + thickness))
+
+        if orient_vertical:
+            available_size = self.get_allocation().width
+        else:
+            available_size = self.get_allocation().height
+        
+        return required_size <= available_size
+
+    def on_label_style_set(self, widget, something):
+        self.reformat_label()
+
+
+    def on_size_allocate(self, widget, allocation):
+        if not self.get_parent():
+            return
+
+        self.popup_dir = self.get_parent().get_orient()
+        
+        orient_vertical = True
+        new_size = allocation.width
+        if self.popup_dir in [gnomeapplet.ORIENT_LEFT]:
+            new_angle = 270
+        elif self.popup_dir in [gnomeapplet.ORIENT_RIGHT]:
+            new_angle = 90
+        else:
+            new_angle = 0
+            orient_vertical = False
+            new_size = allocation.height
+        
+        if new_angle != self.label.get_angle():
+            self.label.set_angle(new_angle)
+
+        if new_size != self.prev_size:
+            self.reformat_label()
+
+        self.prev_size = new_size
+
 
 class HamsterApplet(object):
+    def name_painter(self, column, cell, model, iter):
+        activity_name = model.get_value(iter, 1)
+        description = model.get_value(iter, 5)
+
+        text = """%s""" % activity_name
+        if description:
+            text+= """\n<span style="italic" size="small">%s</span>""" % (description)
+            
+        cell.set_property('markup', text)
+            
+        return
+
+
     def __init__(self, applet):
         self.config = GconfStore.get_instance()
         
         self.applet = applet
         self.applet.set_applet_flags (gnomeapplet.EXPAND_MINOR);
 
+        self.preferences_editor = None
+        self.applet.about = None
+
         self.button = PanelButton()
         
         # load window of activity switcher and todays view
         self.glade = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, "menu.glade"))
         self.window = self.glade.get_widget('hamster-window')
-        
-        # set up drop down menu
-        self.activity_list = self.glade.get_widget('activity-list')
-        self.activity_list.set_model(gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT))
-        self.activity_list.set_text_column(0)
+        self.window.set_keep_above(True)
 
-        # set up autocompletition for the drop-down menu
-        self.activities = gtk.ListStore(gobject.TYPE_STRING)
-        completion = gtk.EntryCompletion()
-        completion.set_model(self.activities)
-        completion.set_text_column(0)
-        completion.set_minimum_key_length(1)
-        self.activity_list.child.set_completion(completion)
+        self.set_dropdown()
 
         # init today's tree
         self.treeview = self.glade.get_widget('today')
         self.treeview.set_tooltip_column(1)
         
         self.treeview.append_column(gtk.TreeViewColumn("Time", gtk.CellRendererText(), text=2))
-        self.treeview.append_column(ExpanderColumn("Name", text = 1))
-        self.treeview.append_column(gtk.TreeViewColumn("", gtk.CellRendererText(), text=3))
+
+        nameColumn = gtk.TreeViewColumn(_("Name"))
+        nameColumn.set_expand(True)
+        nameCell = gtk.CellRendererText()
+        nameColumn.pack_start(nameCell, True)
+        nameCell.set_property("ellipsize", pango.ELLIPSIZE_END)
+        nameColumn.set_cell_data_func(nameCell, self.name_painter)
+        self.treeview.append_column(nameColumn)
+
         
         edit_cell = gtk.CellRendererPixbuf()
         edit_cell.set_property("stock_id", "gtk-edit")
         self.edit_column = gtk.TreeViewColumn("", edit_cell)
         self.treeview.append_column(self.edit_column)
 
+        # DBus Setup
+        try:
+            dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+            name = dbus.service.BusName(HAMSTER_URI, dbus.SessionBus())
+            self.dbusController = HamsterDbusController(bus_name = name)
 
+            # let's also attach our listeners here
+            bus = dbus.SessionBus()
+            bus.add_signal_receiver(self.on_idle_changed,
+                                    dbus_interface="org.gnome.ScreenSaver",
+                                    signal_name="SessionIdleChanged")
+        except dbus.DBusException, e:
+            print "can't init dbus: %s" % e
+    
         # Load today's data, activities and set label
         self.last_activity = None
         self.today = datetime.date.today()
@@ -99,6 +239,9 @@ class HamsterApplet(object):
         self.load_day()
         self.update_label()
 
+        # Hamster DBusController current fact initialising
+        self.__update_fact()
+
         # refresh hamster every 60 seconds to update duration
         gobject.timeout_add_seconds(60, self.refresh_hamster)
 
@@ -125,6 +268,7 @@ class HamsterApplet(object):
             SHARED_DATA_DIR, "Hamster_Applet.xml",
             None, [
             ("about", self.on_about),
+            ("overview", self.show_overview),
             ("preferences", self.show_preferences),
             ])
 
@@ -132,23 +276,85 @@ class HamsterApplet(object):
         self.applet.set_background_widget(self.applet)
 
         self.button.connect('toggled', self.on_toggle)
-        self.button.connect('size_allocate', self.on_applet_size_allocate)
 
         self.button.connect('button_press_event', self.on_button_press)
         self.glade.signal_autoconnect(self)
 
         # init hotkey
         dispatcher.add_handler('keybinding_activated', self.on_keybinding_activated)
-        dispatcher.add_handler('gconf_timeout_changed', self.on_timeout_changed)
-        dispatcher.add_handler('gconf_timeout_enabled_changed', self.on_timeout_enabled_changed)
-  
+
         # init idle check
-        self.timeout_enabled = self.config.get_timeout_enabled()
+        dispatcher.add_handler('gconf_timeout_enabled_changed', self.on_timeout_enabled_changed)
+        self.on_timeout_enabled_changed(None, self.config.get_timeout_enabled())
+        
+        # init nagging timeout
+        if PYNOTIFY:
+            self.notify = Notifier('HamsterApplet', gtk.STOCK_DIALOG_QUESTION, self.button)
+            dispatcher.add_handler('gconf_notify_interval_changed', self.on_notify_interval_changed)
+            self.on_notify_interval_changed(None, self.config.get_notify_interval())
+
+
+    def on_idle_changed(self, state):
+        print "Idle state changed. Idle: ", state
+        # refresh when we are out of idle
+        # (like, instantly after computer has been turned on!
+        if state == 0:
+            self.refresh_hamster() 
+        
+    def set_dropdown(self):
+        # set up drop down menu
+        self.activity_list = self.glade.get_widget('activity-list')
+        self.activity_list.set_model(gtk.ListStore(gobject.TYPE_STRING,
+                                                   gobject.TYPE_STRING,
+                                                   gobject.TYPE_STRING))
+
+        self.activity_list.clear()
+        activity_cell = gtk.CellRendererText()
+        self.activity_list.pack_start(activity_cell, True)
+        self.activity_list.add_attribute(activity_cell, 'text', 0)
+        category_cell = CategoryCell()  
+        self.activity_list.pack_start(category_cell, False)
+        self.activity_list.add_attribute(category_cell, 'text', 1)
+        
+        self.activity_list.set_property("text-column", 2)
+
+
+        # set up autocompletition
+        self.activities = gtk.ListStore(gobject.TYPE_STRING,
+                                        gobject.TYPE_STRING,
+                                        gobject.TYPE_STRING)
+        completion = gtk.EntryCompletion()
+        completion.set_model(self.activities)
+
+        activity_cell = gtk.CellRendererText()
+        completion.pack_start(activity_cell, True)
+        completion.add_attribute(activity_cell, 'text', 0)
+        completion.set_property("text-column", 2)
+
+        category_cell = CategoryCell()  
+        completion.pack_start(category_cell, False)
+        completion.add_attribute(category_cell, 'text', 1)
+
+
+        def match_func(completion, key, iter):
+            model = completion.get_model()
+            text = model.get_value(iter, 2)
+            if text and text.startswith(key):
+                return True
+            return False
+
+        completion.set_match_func(match_func)
+        completion.set_minimum_key_length(1)
+        completion.set_inline_completion(True)
+
+        self.activity_list.child.set_completion(completion)
+        
 
     def on_today_release_event(self, tree, event):
         pointer = event.window.get_pointer() # x, y, flags
         path = tree.get_path_at_pos(pointer[0], pointer[1]) #column, innerx, innery
-        if path[1] == self.edit_column:
+        
+        if path and path[1] == self.edit_column:
             selection = tree.get_selection()
             (model, iter) = selection.get_selected()
             fact_id = model[iter][0]
@@ -177,6 +383,11 @@ class HamsterApplet(object):
             
 
         if self.last_activity and self.last_activity['end_time'] == None:
+            # if we have running task and nagging is enabled
+            # check if maybe it is time to nag
+            if self.notify_interval:
+                self.check_user()
+
             # if we have date change - let's finish previous task and start a new one
             if prev_date and prev_date != self.today: 
                 storage.touch_fact(self.last_activity)
@@ -192,14 +403,35 @@ class HamsterApplet(object):
             delta = datetime.datetime.now() - self.last_activity['start_time']
             duration = delta.seconds /  60
             label = "%s %s" % (self.last_activity['name'], format_duration(duration))
+            self.button.set_text(self.last_activity['name'], format_duration(duration))
             
             self.glade.get_widget('current_activity').set_text(self.last_activity['name'])
             self.glade.get_widget('stop_tracking').set_sensitive(1);
         else:
             label = "%s" % _(u"No activity")
+            self.button.set_text(label, None)
             self.glade.get_widget('stop_tracking').set_sensitive(0);
-        self.button.set_text(label)
         
+        
+        # Hamster DBusController current activity updating
+        self.dbusController.update_activity(label)
+
+    def check_user(self):
+        delta = datetime.datetime.now() - self.last_activity['start_time']
+        duration = delta.seconds /  60
+                 
+        if duration and duration % self.notify_interval == 0:
+            # activity reminder
+            msg = _(u"Are you still working on <b>%s</b>?") % self.last_activity['name']
+            self.notify.msg(msg, self.switch_cb, self.stop_cb)
+
+    def switch_cb(self, n, action):
+        self.__show_toggle(None, not self.button.get_active())	
+
+    def stop_cb(self, n, action):
+        self.on_stop_tracking(None)
+
+
     def load_day(self):
         """sets up today's tree and fills it with records
            returns information about last activity"""
@@ -209,19 +441,30 @@ class HamsterApplet(object):
         if len(day.facts) == 0:
             self.last_activity = None
             self.glade.get_widget("todays_scroll").hide()
-            self.glade.get_widget("no_facts_today").show()
+            
+            self.glade.get_widget("fact_totals").set_text(_("No records today"))
         else:
             self.last_activity = day.facts[len(day.facts) - 1]
             self.glade.get_widget("todays_scroll").show()
-            self.glade.get_widget("no_facts_today").hide()
+            
+            total_string = ""
+            for total in day.totals:
+                total_string += _("%(category)s: %(duration)s, ") % ({'category': total,
+                                                                      'duration': format_duration(day.totals[total])})
+
+            total_string = total_string.rstrip(", ") # trailing slash
+            self.glade.get_widget("fact_totals").set_text(total_string)
    
 
     def refresh_menu(self):
-        #first populate the autocomplete - contains all entries
+        #first populate the autocomplete - contains all entries in lowercase
         self.activities.clear()
-        all_activities = storage.get_activities()
+        all_activities = storage.get_autocomplete_activities()
         for activity in all_activities:
-            self.activities.append([activity['name']])
+            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            self.activities.append([activity['name'],
+                                    activity['category'],
+                                    activity_category])
 
 
         #now populate the menu - contains only categorized entries
@@ -232,13 +475,17 @@ class HamsterApplet(object):
         categorized_activities = storage.get_sorted_activities()
 
         for activity in categorized_activities:
-            item = store.append([activity['name'], activity['id']])
+            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            item = store.append([activity['name'],
+                                 activity['category'],
+                                 activity_category])
 
         # finally add TODO tasks from evolution to both lists
         tasks = hamster.eds.get_eds_tasks()
         for activity in tasks:
-            self.activities.append([activity['name']])
-            store.append([activity['name'], -1])
+            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            self.activities.append([activity['name'],activity['category'],activity_category])
+            store.append([activity['name'], activity['category'], activity_category])
 
         return True
 
@@ -256,6 +503,14 @@ class HamsterApplet(object):
         self.treeview.set_cursor(cur)
 
 
+    def __update_fact(self):
+        """dbus controller current fact updating"""
+        if self.last_activity and self.last_activity['end_time'] == None:
+            self.dbusController.update_fact(self.last_activity["name"])
+        else:
+            self.dbusController.update_fact(_(u'No activity'))
+
+
     def __show_toggle(self, event, is_active):
         """main window display and positioning"""
         self.button.set_active(is_active)
@@ -290,6 +545,11 @@ class HamsterApplet(object):
         else:
             self.activity_list.child.set_text('')
 
+        # doing unstick / stick here, because sometimes while switching
+        # between workplaces window still manages to dissappear
+        self.window.unstick()
+        self.window.stick() #show on all desktops
+
         gobject.idle_add(self._delayed_display)  
         
     def _delayed_display(self):
@@ -360,21 +620,31 @@ class HamsterApplet(object):
         stats_viewer = StatsViewer()
         stats_viewer.show()
 
+    def show_overview(self, menu_item, verb):
+        return self.on_overview(menu_item)
+
     def on_custom_fact(self, menu_item):
         from hamster.add_custom_fact import CustomFactController
         custom_fact = CustomFactController()
         custom_fact.show()
 
     def on_about (self, component, verb):
-        from hamster.about import show_about
-        show_about(self.applet)
+        if self.applet.about:
+            self.applet.about.present()
+        else:
+            from hamster.about import show_about
+            show_about(self.applet)
 
     def show_preferences(self, menu_item, verb):
         from hamster.preferences import PreferencesEditor
 
         dispatcher.dispatch('panel_visible', False)
-        activities_editor = PreferencesEditor()
-        activities_editor.show()
+        
+        if self.preferences_editor and self.preferences_editor.window:
+            self.preferences_editor.window.present()
+        else:
+            self.preferences_editor = PreferencesEditor()
+            self.preferences_editor.show()
     
     """signals"""
     def after_activity_update(self, widget, renames):
@@ -386,27 +656,19 @@ class HamsterApplet(object):
         if date.date() == datetime.date.today():
             self.load_day()
             self.update_label()
+    
+        self.__update_fact()
 
     """global shortcuts"""
     def on_keybinding_activated(self, event, data):
         self.__show_toggle(None, not self.button.get_active())
         
-    def on_timeout_changed(self, event, new_timeout):
-        self.timeout = new_timeout
-        
     def on_timeout_enabled_changed(self, event, enabled):
         # if enabled, set to value, otherwise set to zero, which means disable
         self.timeout_enabled = enabled
 
-    def on_applet_size_allocate(self, widget, event):
-        self.popup_dir = self.applet.get_orient()
-        
-        if self.popup_dir in [gnomeapplet.ORIENT_LEFT]:
-            new_angle = 270
-        elif self.popup_dir in [gnomeapplet.ORIENT_RIGHT]:
-            new_angle = 90
+    def on_notify_interval_changed(self, event, new_interval):
+        if PYNOTIFY and 0 < new_interval < 121:
+            self.notify_interval = new_interval
         else:
-            new_angle = 0
-
-        if new_angle != self.button.label.get_angle():
-            self.button.label.set_angle(new_angle)
+            self.notify_interval = None
