@@ -166,10 +166,17 @@ class PreferencesEditor:
         self.prev_selected_activity = None
         self.prev_selected_category = None
         
+        # disable notification thing if pynotify is not available
+        try:
+            import pynotify
+        except:
+            self.get_widget("notification_preference_frame").hide()
+
 
     def load_config(self):
         self.get_widget("shutdown_track").set_active(self.config.get_stop_on_shutdown())
         self.get_widget("idle_track").set_active(self.config.get_timeout_enabled())
+        self.get_widget("notify_interval").set_value(self.config.get_notify_interval())
         self.get_widget("keybinding").set_text(self.config.get_keybinding())
 
 
@@ -296,9 +303,18 @@ class PreferencesEditor:
         id = tree_row[0]
         
         if id == -1:
-            tree_row[0] = storage.insert_activity(name, category_id)
+            #search for activities with same name
+            activities = storage.get_activities(category_id)
+            for activity in activities:
+                if activity['name'].lower() == name:
+                    self.select_activity(activity['id'])
+                    return False
+            
+            tree_row[0] = storage.add_activity(name, category_id)
         else:
             storage.update_activity(id, name, category_id)
+            
+        return True
 
     # callbacks
     def category_edited_cb(self, cell, path, new_text, model):
@@ -307,7 +323,7 @@ class PreferencesEditor:
         name = model[path][1]
         
         if id == -2:
-            id = storage.insert_category(name)
+            id = storage.add_category(name)
             model[path][0] = id
         elif id > -1:  #ignore unsorted category (id = -1)
             storage.update_category(id, name)
@@ -315,8 +331,15 @@ class PreferencesEditor:
 
     def activity_name_edited_cb(self, cell, path, new_text, model):
         model[path][1] = new_text
-        self.update_activity(model[path])
-        return True
+        res = self.update_activity(model[path])
+        
+        # if update fails on new activity - it's because it is a dupe
+        # so we remove it
+        if not res and model[path][0] == -1: 
+           self.activity_store.remove(model.get_iter(path))
+        
+        return res
+        
 
     def category_changed_cb(self, selection, model):
         """ enables and disables action buttons depending on selected item """
@@ -458,7 +481,10 @@ class PreferencesEditor:
             tree.set_cursor(path, focus_column = self.categoryColumn, start_editing = True)
             #tree.grab_focus()
             #tree.set_cursor(path, start_editing = True)
-                
+
+    def on_preferences_window_key_press(self, widget, event):
+        if event.keyval == gtk.keysyms.Escape:
+            self.window.destroy()     
 
     """button events"""
     def on_add_category_clicked(self, button):
@@ -519,6 +545,23 @@ class PreferencesEditor:
 
     def on_idle_track_toggled(self, checkbox):
         self.config.set_timeout_enabled(checkbox.get_active())
+
+    def on_notify_interval_format_value(self, slider, value):
+        if value <=120:
+            # notify interval slider value label
+            label = _("%(interval_minutes)d minutes" % {'interval_minutes': value})
+        else:
+            # notify interval slider value label
+            label = _("Never")
+        
+        return label
+    
+    def on_notify_interval_value_changed(self, scale):
+        self.config.set_notify_interval(int(scale.get_value()))
     
     def on_keybinding_changed(self, textbox):
         self.config.set_keybinding(textbox.get_text())
+
+    def on_preferences_window_destroy(self, window):
+        self.window = None
+        
