@@ -51,15 +51,18 @@ class Storage(object):
 
     def remove_fact(self, fact_id):
         fact = self.get_fact(fact_id)
-        result = self.__remove_fact(fact_id)
-        self.dispatch('day_updated', fact['start_time'])
-        return result
+        if fact:
+            self.__remove_fact(fact_id)
+            self.dispatch('day_updated', fact['start_time'])
 
+    def get_activities(self, category_id = None):
+        return self.__get_activities(category_id = category_id)
+    
     def get_sorted_activities(self):
         return self.__get_sorted_activities()
         
-    def get_activities(self, category_id = None):
-        return self.__get_activities(category_id = category_id)
+    def get_autocomplete_activities(self):
+        return self.__get_autocomplete_activities()
     
     def get_last_activity(self):
         return self.__get_last_activity()
@@ -90,8 +93,8 @@ class Storage(object):
         self.__update_activity(id, name, category_id)
         self.dispatch('activity_updated', ())
     
-    def insert_activity(self, name, category_id = -1):
-        new_id = self.__insert_activity(name, category_id)
+    def add_activity(self, name, category_id = -1):
+        new_id = self.__add_activity(name, category_id)
         self.dispatch('activity_updated', ())
         return new_id
 
@@ -99,8 +102,8 @@ class Storage(object):
         self.__update_category(id, name)
         self.dispatch('activity_updated', ())
 
-    def insert_category(self, name):
-        res = self.__insert_category(name)
+    def add_category(self, name):
+        res = self.__add_category(name)
         self.dispatch('activity_updated', ())
         return res
 
